/* -------------------------------------------------------------- */
/* Demonstration program for multi-threaded EMM model evaluation  */
/*                                                                */
/* The model is evaluated NVAL times in a parallelized loop.      */
/* Contributions to the model can be turned on and off using      */
/* control parameters. These can also used to set the desired     */
/* degree of the internal and external field                      */
/* The number of threads (processors) can be specified by         */
/* setting the shell variable OMP_NUM_THREADS.                    */
/* In the bash shell, for example, set 8 threads by the statement */
/* 'declare -x OMP_NUM_THREADS=8'                                 */
/*                                          Stefan Maus, Nov-2006 */
/* -------------------------------------------------------------- */

#include <stdio.h>
#include <math.h>

#include "./modeldef.h"


#define MAGNETIC_MODEL       11      /* 1 = WMM-2005 for testing */
                                     /* 10 = EMM-06 */
                                     /* 11 = EMM-07 */
                                     /* 20 = POMME-4.0  (const secular acc, degree 36 */
                                     /* 21 = POMME-4.1  (const secular acc, degree 100 */
                                     /* 22 = POMME-4.2  (const secular acc, degree 720 */
                                     /* 25 = POMME-4.0s (split secular acc, degree 36 */
                                     /* 26 = POMME-4.1s (split secular acc, degree 100 */
                                     /* 27 = POMME-4.2s (split secular acc, degree 720 */
                                     /* 30 = POMME-5.0 (split secular acc, degree 60 */

#if MAGNETIC_MODEL == 1
#define MAINFNAME       "./coeff/WMM-2005_EMM-format.cof"
#elif MAGNETIC_MODEL == 10
#define MAINFNAME       "./coeff/EMM-06.cof"
#elif MAGNETIC_MODEL == 11
#define MAINFNAME       "./coeff/EMM-07.cof"
#elif MAGNETIC_MODEL == 20
#define MAINFNAME       "./coeff/pomme-4.0.cof"
#elif MAGNETIC_MODEL == 21
#define MAINFNAME       "./coeff/pomme-4.1.cof"
#elif MAGNETIC_MODEL == 22
#define MAINFNAME       "./coeff/pomme-4.2.cof"
#elif MAGNETIC_MODEL == 25
#define MAINFNAME       "./coeff/pomme-4.0s.cof"
#elif MAGNETIC_MODEL == 26
#define MAINFNAME       "./coeff/pomme-4.1s.cof"
#elif MAGNETIC_MODEL == 27
#define MAINFNAME       "./coeff/pomme-4.2s.cof"
#elif MAGNETIC_MODEL == 30
#define MAINFNAME       "./coeff/pomme-5.0.cof"
#endif

#define USE_OPENMP 1            /* 0=single thread, 1=parallel */

#if USE_OPENMP
#include <omp.h>
#endif

#define SM2GEOCF        "./coeff/sm2geo_coeff_3_6"
#define GSM2GEOCF       "./coeff/gsm2geo_coeff_3_6"


#define NVAL 10                  /* number of test values to compute */


static tcontrol control;        /* array of user-control parameters */
static tmodel model;            /* array containing model coefficients */

extern void mainfield_open_file_mp (char mainfname[], char sm2geo_fname[], char gsm2geo_fname[], int *control,
				   int ncontrol, tmodel *model,
				   int model_nbytes);

extern void mainfield_geodetic_mp (int control[MAXCONTROL], tmodel *model,
                                   double fday, double geod_alt, double geod_lat, double lon,
                                   double est, double ist, double imf_by,
                                   double *geod_x, double *geod_y, double *geod_z, 
                                   double *h, double *f, double *incl, double *decl);


static  double lon[NVAL], geod_lat[NVAL], geod_alt[NVAL], fday[NVAL];  /* Position and time */
static  double E_st[NVAL], I_st[NVAL], IMF_By[NVAL];                   /* Magnetic indices for time varying external field */
static  double x[NVAL], y[NVAL], z[NVAL],  h[NVAL], f[NVAL], incl[NVAL], decl[NVAL]; /* Resulting field */


int main(void)
{
  int i;

  printf("opening %s\n", MAINFNAME);
  mainfield_open_file_mp(MAINFNAME, SM2GEOCF, GSM2GEOCF, control, 13, &model, sizeof(tmodel));

  /* mainfield_open_file_mp initializes the control parameters. You can now modify these: */

  control[POS_0] =         720;   /* default: model.deg_int, 0=off */
  control[POS_1] =         16;    /* default: model.deg_1st, 0=off */
  control[POS_2] =         0;     /* default: model.deg_2nd, 0=off */
  control[POS_3] =         0;     /* default: model.deg_2nd_plus, 0=off */

  control[POS_IMFBY] =     1;     /* IMF-By dependent field: 1=on, 0=off */
  control[POS_IMFBY_IND] = 1;     /* Induction by IMF-By dependent field: 1=on, 0=off */

  control[POS_EST] =       1;     /* Est dependent field: 1=on, 0=off */
  control[POS_IST] =       1;     /* Ist dependent field: 1=on, 0=off */
  control[POS_EST_FAC] =   1;     /* 1 = use model Est factor, 0 = use factor 1.0 */

  control[POS_SM] =        2;     /* degree of SM field, default = model.deg_sm, 0=off */
  control[POS_SM_IND] =    2;     /* degree of field induced by SM field, default = model.deg_sm, 0=off */
  control[POS_GSM] =       2;     /* degree of GSM field, default = model.deg_gsm, 0=off */
  control[POS_GSM_IND] =   2;     /* degree of field induced by GSM field, default = model.deg_gsm, 0=off */

#if USE_OPENMP
#pragma omp parallel default(none) \
                     private(i)\
                     shared(control,model,E_st,I_st,IMF_By,lon,geod_lat,geod_alt,fday,x,y,z,h,f,incl,decl)

{
#pragma omp for nowait
#endif

  for (i=0; i<NVAL; i++)
    {

      /* The following are just arbitrary test values */

      fday[i]   = 300.1 + i*33.07; /* floating point days since 00:00:00 on 0/0/2000 */
      geod_alt[i] = i*5.0;      /* in km */
      geod_lat[i] = 50+i*2.0;      /* in floating point degrees */
      lon[i] = i*20.0;           /* in floating point degrees */

      IMF_By[i] = -5.0 + i/10.0;
      E_st[i]   = -1.2*i;
      I_st[i]   = 0.3*E_st[i];

      mainfield_geodetic_mp(control, &model, fday[i], geod_alt[i], geod_lat[i], lon[i], E_st[i], I_st[i], 
                            IMF_By[i], &x[i], &y[i], &z[i], &h[i], &f[i], &incl[i], &decl[i]);                                         

    }
#if USE_OPENMP
       } /* end of parallel REGION */
#endif
 printf("\n\n");
 printf("    fday     lon     lat     alt    E_st    I_st  IMF_By     |          X          Y          Z          H          F       Incl       Decl\n");
 printf("--------------------------------------------------------------------------------------------------------------------------------------------\n");

  for (i=0; i<NVAL; i++)
    {
      printf("%8.1f %7.1f %7.1f %7.1f %7.1f %7.1f %7.1f     |  %10.2f %10.2f %10.2f %10.2f %10.2f %10.5f %10.5f\n",
             fday[i], lon[i], geod_lat[i], geod_alt[i], E_st[i], I_st[i], IMF_By[i],
             x[i], y[i], z[i], h[i], f[i], incl[i], decl[i]);
    }

} /* End. */
